@@ -1,10 +1,12 @@
 """Configuration base classes."""
 
 from abc import ABC, abstractmethod
-from typing import Mapping
+from typing import Any, Iterator, Mapping, Optional, Type, TypeVar
 
 from ..core.error import BaseError
 
+InjectType = TypeVar("InjectType")
+
 
 class ConfigError(BaseError):
     """A base exception for all configuration errors."""
@@ -14,11 +16,11 @@ class SettingsError(ConfigError):
     """The base exception raised by `BaseSettings` implementations."""
 
 
-class BaseSettings(Mapping[str, object]):
+class BaseSettings(Mapping[str, Any]):
     """Base settings class."""
 
     @abstractmethod
-    def get_value(self, *var_names, default=None):
+    def get_value(self, *var_names, default: Optional[Any] = None) -> Any:
         """Fetch a setting.
 
         Args:
@@ -30,7 +32,7 @@ class BaseSettings(Mapping[str, object]):
 
         """
 
-    def get_bool(self, *var_names, default=None) -> bool:
+    def get_bool(self, *var_names, default: Optional[bool] = None) -> Optional[bool]:
         """Fetch a setting as a boolean value.
 
         Args:
@@ -40,9 +42,10 @@ class BaseSettings(Mapping[str, object]):
         value = self.get_value(*var_names, default)
         if value is not None:
             value = bool(value and value not in ("false", "False", "0"))
+
         return value
 
-    def get_int(self, *var_names, default=None) -> int:
+    def get_int(self, *var_names, default: Optional[int] = None) -> Optional[int]:
         """Fetch a setting as an integer value.
 
         Args:
@@ -52,9 +55,10 @@ class BaseSettings(Mapping[str, object]):
         value = self.get_value(*var_names, default)
         if value is not None:
             value = int(value)
+
         return value
 
-    def get_str(self, *var_names, default=None) -> str:
+    def get_str(self, *var_names, default: Optional[str] = None) -> Optional[str]:
         """Fetch a setting as a string value.
 
         Args:
@@ -64,21 +68,22 @@ class BaseSettings(Mapping[str, object]):
         value = self.get_value(*var_names, default=default)
         if value is not None:
             value = str(value)
+
         return value
 
     @abstractmethod
-    def __iter__(self):
+    def __iter__(self) -> Iterator:
         """Iterate settings keys."""
 
     def __getitem__(self, index):
         """Fetch as an array index."""
         if not isinstance(index, str):
-            raise TypeError("Index must be a string")
+            raise TypeError(f"Index {index} must be a string")
         missing = object()
         result = self.get_value(index, default=missing)
         if result is missing:
             raise KeyError("Undefined index: {}".format(index))
-        return self.get_value(index)
+        return result
 
     @abstractmethod
     def __len__(self):
@@ -89,7 +94,7 @@ class BaseSettings(Mapping[str, object]):
         """Produce a copy of the settings instance."""
 
     @abstractmethod
-    def extend(self, other: Mapping[str, object]) -> "BaseSettings":
+    def extend(self, other: Mapping[str, Any]) -> "BaseSettings":
         """Merge another mapping to produce a new settings instance."""
 
     def __repr__(self) -> str:
@@ -98,21 +103,19 @@ class BaseSettings(Mapping[str, object]):
         return "<{}({})>".format(self.__class__.__name__, ", ".join(items))
 
 
-class InjectorError(ConfigError):
-    """The base exception raised by `BaseInjector` implementations."""
+class InjectionError(ConfigError):
+    """The base exception raised by Injector and Provider implementations."""
 
 
 class BaseInjector(ABC):
     """Base injector class."""
 
     @abstractmethod
-    async def inject(
+    def inject(
         self,
-        base_cls: type,
-        settings: Mapping[str, object] = None,
-        *,
-        required: bool = True
-    ) -> object:
+        base_cls: Type[InjectType],
+        settings: Optional[Mapping[str, Any]] = None,
+    ) -> InjectType:
         """
         Get the provided instance of a given class identifier.
 
@@ -125,17 +128,33 @@ class BaseInjector(ABC):
 
         """
 
+    @abstractmethod
+    def inject_or(
+        self,
+        base_cls: Type[InjectType],
+        settings: Optional[Mapping[str, Any]] = None,
+        default: Optional[InjectType] = None,
+    ) -> Optional[InjectType]:
+        """
+        Get the provided instance of a given class identifier or default if not found.
+
+        Args:
+            base_cls: The base class to retrieve an instance of
+            settings: An optional dict providing configuration to the provider
+            default: default return value if no instance is found
+
+        Returns:
+            An instance of the base class, or None
+
+        """
+
     @abstractmethod
     def copy(self) -> "BaseInjector":
         """Produce a copy of the injector instance."""
 
 
-class ProviderError(ConfigError):
-    """The base exception raised by `BaseProvider` implementations."""
-
-
 class BaseProvider(ABC):
     """Base provider class."""
 
-    async def provide(self, settings: BaseSettings, injector: BaseInjector):
+    def provide(self, settings: BaseSettings, injector: BaseInjector):
         """Provide the object instance given a config and injector."""
