@@ -1,34 +1,28 @@
 from io import StringIO
 
-import asynctest
-from asynctest import TestCase as AsyncTestCase
-from asynctest import mock as async_mock
+import mock as async_mock
+from async_case import IsolatedAsyncioTestCase
 
 from ...admin.base_server import BaseAdminServer
 from ...config.base_context import ContextBuilder
 from ...config.injection_context import InjectionContext
 from ...connections.models.conn_record import ConnRecord
 from ...connections.models.connection_target import ConnectionTarget
-from ...connections.models.diddoc import (
-    DIDDoc,
-    PublicKey,
-    PublicKeyType,
-    Service,
-)
+from ...connections.models.diddoc import DIDDoc, PublicKey, PublicKeyType, Service
 from ...core.event_bus import EventBus, MockEventBus
 from ...core.in_memory import InMemoryProfileManager
 from ...core.profile import ProfileManager
 from ...core.protocol_registry import ProtocolRegistry
+from ...multitenant.base import BaseMultitenantManager
+from ...multitenant.manager import MultitenantManager
 from ...protocols.coordinate_mediation.mediation_invite_store import (
     MediationInviteRecord,
-    MediationInviteStore,
 )
 from ...protocols.coordinate_mediation.v1_0.models.mediation_record import (
     MediationRecord,
 )
-from ...resolver.did_resolver import DIDResolver, DIDResolverRegistry
-from ...multitenant.base import BaseMultitenantManager
-from ...multitenant.manager import MultitenantManager
+from ...protocols.out_of_band.v1_0.models.oob_record import OobRecord
+from ...resolver.did_resolver import DIDResolver
 from ...storage.base import BaseStorage
 from ...storage.error import StorageNotFoundError
 from ...transport.inbound.message import InboundMessage
@@ -37,14 +31,13 @@ from ...transport.outbound.base import OutboundDeliveryError
 from ...transport.outbound.manager import QueuedOutboundMessage
 from ...transport.outbound.message import OutboundMessage
 from ...transport.outbound.status import OutboundSendStatus
-from ...transport.wire_format import BaseWireFormat
 from ...transport.pack_format import PackWireFormat
+from ...transport.wire_format import BaseWireFormat
 from ...utils.stats import Collector
 from ...version import __version__
 from ...wallet.base import BaseWallet
-from ...wallet.key_type import KeyType
-from ...wallet.did_method import DIDMethod
-
+from ...wallet.did_method import SOV, DIDMethods
+from ...wallet.key_type import ED25519
 from .. import conductor as test_module
 
 
@@ -94,7 +87,8 @@ class StubContextBuilder(ContextBuilder):
         context.injector.bind_instance(ProfileManager, InMemoryProfileManager())
         context.injector.bind_instance(ProtocolRegistry, ProtocolRegistry())
         context.injector.bind_instance(BaseWireFormat, self.wire_format)
-        context.injector.bind_instance(DIDResolver, DIDResolver(DIDResolverRegistry()))
+        context.injector.bind_instance(DIDMethods, DIDMethods())
+        context.injector.bind_instance(DIDResolver, DIDResolver([]))
         context.injector.bind_instance(EventBus, MockEventBus())
         return context
 
@@ -106,7 +100,7 @@ class StubCollectorContextBuilder(StubContextBuilder):
         return context
 
 
-class TestConductor(AsyncTestCase, Config, TestDIDs):
+class TestConductor(IsolatedAsyncioTestCase, Config, TestDIDs):
     async def test_startup(self):
         builder: ContextBuilder = StubContextBuilder(self.test_settings)
         conductor = test_module.Conductor(builder)
@@ -120,7 +114,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as mock_logger, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -133,8 +127,8 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
 
             wallet = session.inject(BaseWallet)
             await wallet.create_public_did(
-                DIDMethod.SOV,
-                KeyType.ED25519,
+                SOV,
+                ED25519,
             )
 
             mock_inbound_mgr.return_value.setup.assert_awaited_once()
@@ -188,14 +182,14 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as mock_logger, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
             mock_outbound_mgr.return_value.registered_transports = {}
-            mock_outbound_mgr.return_value.enqueue_message = async_mock.CoroutineMock()
-            mock_outbound_mgr.return_value.start = async_mock.CoroutineMock()
-            mock_outbound_mgr.return_value.stop = async_mock.CoroutineMock()
+            mock_outbound_mgr.return_value.enqueue_message = async_mock.AsyncMock()
+            mock_outbound_mgr.return_value.start = async_mock.AsyncMock()
+            mock_outbound_mgr.return_value.stop = async_mock.AsyncMock()
             await conductor.setup()
 
             mock_inbound_mgr.return_value.setup.assert_awaited_once()
@@ -228,7 +222,6 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as mock_outbound_mgr, async_mock.patch.object(
             test_module, "LoggingConfigurator", autospec=True
         ) as mock_logger:
-
             mock_inbound_mgr.return_value.sessions = ["dummy"]
             mock_outbound_mgr.return_value.outbound_buffer = [
                 async_mock.MagicMock(state=QueuedOutboundMessage.STATE_ENCODE),
@@ -269,7 +262,6 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         with async_mock.patch.object(
             conductor.dispatcher, "queue_message", autospec=True
         ) as mock_dispatch_q:
-
             message_body = "{}"
             receipt = MessageReceipt(direct_response_mode="snail mail")
             message = InboundMessage(message_body, receipt)
@@ -464,8 +456,8 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             test_module, "OutboundTransportManager", async_mock.MagicMock()
         ) as mock_outbound_mgr:
             mock_outbound_mgr.return_value = async_mock.MagicMock(
-                setup=async_mock.CoroutineMock(),
-                enqueue_message=async_mock.CoroutineMock(),
+                setup=async_mock.AsyncMock(),
+                enqueue_message=async_mock.AsyncMock(),
             )
 
             payload = "{}"
@@ -485,7 +477,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
                 conductor.dispatcher, "run_task", async_mock.MagicMock()
             ) as mock_run_task:
                 mock_conn_mgr.return_value.get_connection_targets = (
-                    async_mock.CoroutineMock()
+                    async_mock.AsyncMock()
                 )
                 mock_run_task.side_effect = test_module.ConnectionManagerError()
                 await conductor.queue_outbound(conductor.root_profile, message)
@@ -527,7 +519,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         with async_mock.patch.object(
             conductor.dispatcher, "run_task", async_mock.MagicMock()
         ) as mock_dispatch_run, async_mock.patch.object(
-            conductor, "queue_outbound", async_mock.CoroutineMock()
+            conductor, "queue_outbound", async_mock.AsyncMock()
         ) as mock_queue, async_mock.patch.object(
             conductor.admin_server, "notify_fatal_error", async_mock.MagicMock()
         ) as mock_notify:
@@ -567,7 +559,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as mock_dispatch_run, async_mock.patch.object(
             conductor.admin_server, "notify_fatal_error", async_mock.MagicMock()
         ) as mock_notify:
-            conn_mgr.return_value.get_connection_targets = async_mock.CoroutineMock()
+            conn_mgr.return_value.get_connection_targets = async_mock.AsyncMock()
             mock_dispatch_run.side_effect = test_module.LedgerConfigError(
                 "No such ledger"
             )
@@ -602,8 +594,8 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         session = await conductor.root_profile.session()
         wallet = session.inject(BaseWallet)
         await wallet.create_public_did(
-            DIDMethod.SOV,
-            KeyType.ED25519,
+            SOV,
+            ED25519,
         )
 
         with async_mock.patch.object(
@@ -613,7 +605,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as admin_stop, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -646,8 +638,8 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         session = await conductor.root_profile.session()
         wallet = session.inject(BaseWallet)
         await wallet.create_public_did(
-            DIDMethod.SOV,
-            KeyType.ED25519,
+            SOV,
+            ED25519,
         )
 
         with async_mock.patch.object(
@@ -661,15 +653,15 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as conn_mgr, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
             admin_start.side_effect = KeyError("trouble")
-            oob_mgr.return_value.create_invitation = async_mock.CoroutineMock(
+            oob_mgr.return_value.create_invitation = async_mock.AsyncMock(
                 side_effect=KeyError("double trouble")
             )
-            conn_mgr.return_value.create_invitation = async_mock.CoroutineMock(
+            conn_mgr.return_value.create_invitation = async_mock.AsyncMock(
                 side_effect=KeyError("triple trouble")
             )
             await conductor.start()
@@ -704,7 +696,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as mock_mgr, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ), async_mock.patch.object(
@@ -718,11 +710,11 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             session = await conductor.root_profile.session()
             wallet = session.inject(BaseWallet)
             await wallet.create_public_did(
-                DIDMethod.SOV,
-                KeyType.ED25519,
+                SOV,
+                ED25519,
             )
 
-            mock_mgr.return_value.create_static_connection = async_mock.CoroutineMock()
+            mock_mgr.return_value.create_static_connection = async_mock.AsyncMock()
             await conductor.start()
             mock_mgr.return_value.create_static_connection.assert_awaited_once()
 
@@ -739,14 +731,14 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             test_module, "OutboundTransportManager", autospec=True
         ) as mock_outbound_mgr:
             mock_intx_mgr.return_value = async_mock.MagicMock(
-                setup=async_mock.CoroutineMock(),
-                start=async_mock.CoroutineMock(side_effect=KeyError("trouble")),
+                setup=async_mock.AsyncMock(),
+                start=async_mock.AsyncMock(side_effect=KeyError("trouble")),
             )
             mock_outbound_mgr.return_value.registered_transports = {
                 "test": async_mock.MagicMock(schemes=["http"])
             }
             await conductor.setup()
-            mock_mgr.return_value.create_static_connection = async_mock.CoroutineMock()
+            mock_mgr.return_value.create_static_connection = async_mock.AsyncMock()
             with self.assertRaises(KeyError):
                 await conductor.start()
 
@@ -761,12 +753,12 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             test_module, "OutboundTransportManager"
         ) as mock_outx_mgr:
             mock_outx_mgr.return_value = async_mock.MagicMock(
-                setup=async_mock.CoroutineMock(),
-                start=async_mock.CoroutineMock(side_effect=KeyError("trouble")),
+                setup=async_mock.AsyncMock(),
+                start=async_mock.AsyncMock(side_effect=KeyError("trouble")),
                 registered_transports={"test": async_mock.MagicMock(schemes=["http"])},
             )
             await conductor.setup()
-            mock_mgr.return_value.create_static_connection = async_mock.CoroutineMock()
+            mock_mgr.return_value.create_static_connection = async_mock.AsyncMock()
             with self.assertRaises(KeyError):
                 await conductor.start()
 
@@ -781,14 +773,14 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             test_module, "OutboundTransportManager"
         ) as mock_outx_mgr:
             mock_outx_mgr.return_value = async_mock.MagicMock(
-                setup=async_mock.CoroutineMock(),
-                start=async_mock.CoroutineMock(side_effect=KeyError("trouble")),
-                stop=async_mock.CoroutineMock(),
+                setup=async_mock.AsyncMock(),
+                start=async_mock.AsyncMock(side_effect=KeyError("trouble")),
+                stop=async_mock.AsyncMock(),
                 registered_transports={},
-                enqueue_message=async_mock.CoroutineMock(),
+                enqueue_message=async_mock.AsyncMock(),
             )
             await conductor.setup()
-            mock_mgr.return_value.create_static_connection = async_mock.CoroutineMock()
+            mock_mgr.return_value.create_static_connection = async_mock.AsyncMock()
             with self.assertRaises(KeyError):
                 await conductor.start()
 
@@ -799,8 +791,9 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         message_body = "{}"
         receipt = MessageReceipt(direct_response_mode="snail mail")
         message = InboundMessage(message_body, receipt)
+        exc = KeyError("sample exception")
         mock_task = async_mock.MagicMock(
-            exc_info=(KeyError, KeyError("sample exception"), "..."),
+            exc_info=(type(exc), exc, exc.__traceback__),
             ident="abc",
             timing={
                 "queued": 1234567890,
@@ -831,12 +824,9 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         message_body = "{}"
         receipt = MessageReceipt(direct_response_mode="snail mail")
         message = InboundMessage(message_body, receipt)
+        exc = test_module.LedgerTransactionError("Ledger is wobbly")
         mock_task = async_mock.MagicMock(
-            exc_info=(
-                test_module.LedgerTransactionError,
-                test_module.LedgerTransactionError("Ledger is wobbly"),
-                "...",
-            ),
+            exc_info=(type(exc), exc, exc.__traceback__),
             ident="abc",
             timing={
                 "queued": 1234567890,
@@ -876,7 +866,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ) as captured, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ), async_mock.patch.object(
@@ -890,8 +880,8 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             session = await conductor.root_profile.session()
             wallet = session.inject(BaseWallet)
             await wallet.create_public_did(
-                DIDMethod.SOV,
-                KeyType.ED25519,
+                SOV,
+                ED25519,
             )
 
             await conductor.start()
@@ -917,12 +907,12 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             test_module,
             "MediationManager",
             return_value=async_mock.MagicMock(
-                clear_default_mediator=async_mock.CoroutineMock()
+                clear_default_mediator=async_mock.AsyncMock()
             ),
         ) as mock_mgr, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -947,10 +937,10 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
             test_module,
             "MediationManager",
             return_value=async_mock.MagicMock(
-                set_default_mediator_by_id=async_mock.CoroutineMock()
+                set_default_mediator_by_id=async_mock.AsyncMock()
             ),
         ) as mock_mgr, async_mock.patch.object(
-            MediationRecord, "retrieve_by_id", async_mock.CoroutineMock()
+            MediationRecord, "retrieve_by_id", async_mock.AsyncMock()
         ), async_mock.patch.object(
             test_module,
             "LOGGER",
@@ -962,7 +952,7 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         ), async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -986,13 +976,13 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
         with async_mock.patch.object(
             MediationRecord,
             "retrieve_by_id",
-            async_mock.CoroutineMock(side_effect=Exception()),
+            async_mock.AsyncMock(side_effect=Exception()),
         ), async_mock.patch.object(
             test_module, "LOGGER"
         ) as mock_logger, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -1064,11 +1054,11 @@ class TestConductor(AsyncTestCase, Config, TestDIDs):
 
             multitenant_mgr._profiles.put(
                 "test1",
-                async_mock.MagicMock(close=async_mock.CoroutineMock()),
+                async_mock.MagicMock(close=async_mock.AsyncMock()),
             )
             multitenant_mgr._profiles.put(
                 "test2",
-                async_mock.MagicMock(close=async_mock.CoroutineMock()),
+                async_mock.MagicMock(close=async_mock.AsyncMock()),
             )
 
             await conductor.stop()
@@ -1084,14 +1074,12 @@ def get_invite_store_mock(
     used_invite = MediationInviteRecord(invite_string, used=True)
 
     return async_mock.MagicMock(
-        get_mediation_invite_record=async_mock.CoroutineMock(
-            return_value=unused_invite
-        ),
-        mark_default_invite_as_used=async_mock.CoroutineMock(return_value=used_invite),
+        get_mediation_invite_record=async_mock.AsyncMock(return_value=unused_invite),
+        mark_default_invite_as_used=async_mock.AsyncMock(return_value=used_invite),
     )
 
 
-class TestConductorMediationSetup(AsyncTestCase, Config):
+class TestConductorMediationSetup(IsolatedAsyncioTestCase, Config):
     """
     Test related with setting up mediation from given arguments or stored invitation.
     """
@@ -1106,12 +1094,12 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
 
         return builder
 
-    @asynctest.patch.object(
+    @async_mock.patch.object(
         test_module,
         "MediationInviteStore",
         return_value=get_invite_store_mock("test-invite"),
     )
-    @asynctest.patch.object(test_module.ConnectionInvitation, "from_url")
+    @async_mock.patch.object(test_module.ConnectionInvitation, "from_url")
     async def test_mediator_invitation_0160(self, mock_from_url, _):
         conductor = test_module.Conductor(
             self.__get_mediator_config("test-invite", True)
@@ -1131,17 +1119,17 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
             "ConnectionManager",
             async_mock.MagicMock(
                 return_value=async_mock.MagicMock(
-                    receive_invitation=async_mock.CoroutineMock(
+                    receive_invitation=async_mock.AsyncMock(
                         return_value=mock_conn_record
                     )
                 )
             ),
         ) as mock_mgr, async_mock.patch.object(
-            mock_conn_record, "metadata_set", async_mock.CoroutineMock()
+            mock_conn_record, "metadata_set", async_mock.AsyncMock()
         ), async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -1150,12 +1138,12 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
             mock_from_url.assert_called_once_with("test-invite")
             mock_mgr.return_value.receive_invitation.assert_called_once()
 
-    @asynctest.patch.object(
+    @async_mock.patch.object(
         test_module,
         "MediationInviteStore",
         return_value=get_invite_store_mock("test-invite"),
     )
-    @asynctest.patch.object(test_module.InvitationMessage, "from_url")
+    @async_mock.patch.object(test_module.InvitationMessage, "from_url")
     async def test_mediator_invitation_0434(self, mock_from_url, _):
         conductor = test_module.Conductor(
             self.__get_mediator_config("test-invite", False)
@@ -1167,7 +1155,9 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
                 "test": async_mock.MagicMock(schemes=["http"])
             }
             await conductor.setup()
-
+        conductor.root_profile.context.update_settings(
+            {"mediation.connections_invite": False}
+        )
         conn_record = ConnRecord(
             invitation_key="3Dn1SJNPaCXcvvJvSbsFWP2xaCjMom3can8CQNhWrTRx",
             their_label="Hello",
@@ -1176,30 +1166,37 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         )
         conn_record.accept = ConnRecord.ACCEPT_MANUAL
         await conn_record.save(await conductor.root_profile.session())
+        invitation = test_module.InvitationMessage()
+        oob_record = OobRecord(
+            invitation=invitation,
+            invi_msg_id=invitation._id,
+            role=OobRecord.ROLE_RECEIVER,
+            connection_id=conn_record.connection_id,
+            state=OobRecord.STATE_INITIAL,
+        )
         with async_mock.patch.object(
             test_module,
             "OutOfBandManager",
             async_mock.MagicMock(
                 return_value=async_mock.MagicMock(
-                    receive_invitation=async_mock.CoroutineMock(
-                        return_value=conn_record
-                    )
+                    receive_invitation=async_mock.AsyncMock(return_value=oob_record)
                 )
             ),
         ) as mock_mgr, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
+            assert not conductor.root_profile.settings["mediation.connections_invite"]
             await conductor.start()
             await conductor.stop()
             mock_from_url.assert_called_once_with("test-invite")
             mock_mgr.return_value.receive_invitation.assert_called_once()
 
-    @asynctest.patch.object(test_module, "MediationInviteStore")
-    @asynctest.patch.object(test_module.ConnectionInvitation, "from_url")
+    @async_mock.patch.object(test_module, "MediationInviteStore")
+    @async_mock.patch.object(test_module.ConnectionInvitation, "from_url")
     async def test_mediation_invitation_should_use_stored_invitation(
         self, patched_from_url, patched_invite_store
     ):
@@ -1227,23 +1224,23 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         patched_invite_store.return_value = mocked_store
 
         connection_manager_mock = async_mock.MagicMock(
-            receive_invitation=async_mock.CoroutineMock(return_value=mock_conn_record)
+            receive_invitation=async_mock.AsyncMock(return_value=mock_conn_record)
         )
         mock_mediation_manager = async_mock.MagicMock(
-            clear_default_mediator=async_mock.CoroutineMock()
+            clear_default_mediator=async_mock.AsyncMock()
         )
 
         # when
         with async_mock.patch.object(
             test_module, "ConnectionManager", return_value=connection_manager_mock
         ), async_mock.patch.object(
-            mock_conn_record, "metadata_set", async_mock.CoroutineMock()
+            mock_conn_record, "metadata_set", async_mock.AsyncMock()
         ), async_mock.patch.object(
             test_module, "MediationManager", return_value=mock_mediation_manager
         ), async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -1257,8 +1254,8 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
             patched_from_url.assert_called_with(invite_string)
             mock_mediation_manager.clear_default_mediator.assert_called_once()
 
-    @asynctest.patch.object(test_module, "MediationInviteStore")
-    @asynctest.patch.object(test_module, "ConnectionManager")
+    @async_mock.patch.object(test_module, "MediationInviteStore")
+    @async_mock.patch.object(test_module, "ConnectionManager")
     async def test_mediation_invitation_should_not_create_connection_for_old_invitation(
         self, patched_connection_manager, patched_invite_store
     ):
@@ -1280,13 +1277,13 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         patched_invite_store.return_value = invite_store_mock
 
         connection_manager_mock = async_mock.MagicMock(
-            receive_invitation=async_mock.CoroutineMock()
+            receive_invitation=async_mock.AsyncMock()
         )
         patched_connection_manager.return_value = connection_manager_mock
         with async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -1300,7 +1297,7 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
             )
             connection_manager_mock.receive_invitation.assert_not_called()
 
-    @asynctest.patch.object(
+    @async_mock.patch.object(
         test_module,
         "MediationInviteStore",
         return_value=get_invite_store_mock("test-invite"),
@@ -1326,7 +1323,7 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         ) as mock_logger, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
+            async_mock.AsyncMock(
                 return_value=async_mock.MagicMock(value=f"v{__version__}")
             ),
         ):
@@ -1344,9 +1341,9 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         with async_mock.patch.object(
             test_module,
             "load_multiple_genesis_transactions_from_config",
-            async_mock.CoroutineMock(),
+            async_mock.AsyncMock(),
         ) as mock_multiple_genesis_load, async_mock.patch.object(
-            test_module, "get_genesis_transactions", async_mock.CoroutineMock()
+            test_module, "get_genesis_transactions", async_mock.AsyncMock()
         ) as mock_genesis_load, async_mock.patch.object(
             test_module, "OutboundTransportManager", autospec=True
         ) as mock_outbound_mgr:
@@ -1363,7 +1360,7 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         conductor = test_module.Conductor(builder)
 
         with async_mock.patch.object(
-            test_module, "get_genesis_transactions", async_mock.CoroutineMock()
+            test_module, "get_genesis_transactions", async_mock.AsyncMock()
         ) as mock_genesis_load, async_mock.patch.object(
             test_module, "OutboundTransportManager", autospec=True
         ) as mock_outbound_mgr:
@@ -1386,9 +1383,7 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         ) as mock_logger, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(
-                return_value=async_mock.MagicMock(value=f"v0.6.0")
-            ),
+            async_mock.AsyncMock(return_value=async_mock.MagicMock(value=f"v0.6.0")),
         ):
             mock_outbound_mgr.return_value.registered_transports = {
                 "test": async_mock.MagicMock(schemes=["http"])
@@ -1399,8 +1394,8 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
 
             wallet = session.inject(BaseWallet)
             await wallet.create_public_did(
-                DIDMethod.SOV,
-                KeyType.ED25519,
+                SOV,
+                ED25519,
             )
 
             mock_inbound_mgr.return_value.setup.assert_awaited_once()
@@ -1425,7 +1420,7 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
         ) as mock_logger, async_mock.patch.object(
             BaseStorage,
             "find_record",
-            async_mock.CoroutineMock(side_effect=StorageNotFoundError()),
+            async_mock.AsyncMock(side_effect=StorageNotFoundError()),
         ):
             mock_outbound_mgr.return_value.registered_transports = {
                 "test": async_mock.MagicMock(schemes=["http"])
@@ -1436,8 +1431,8 @@ class TestConductorMediationSetup(AsyncTestCase, Config):
 
             wallet = session.inject(BaseWallet)
             await wallet.create_public_did(
-                DIDMethod.SOV,
-                KeyType.ED25519,
+                SOV,
+                ED25519,
             )
 
             mock_inbound_mgr.return_value.setup.assert_awaited_once()
