@@ -4,7 +4,6 @@ from unittest import TestCase
 
 
 class TestTraceDecorator(TestCase):
-
     target_api = "http://example.com/api/trace/"
     full_thread_api = False
     target_msg = TRACE_MESSAGE_TARGET
@@ -20,15 +19,14 @@ class TestTraceDecorator(TestCase):
     outcome = "OK ..."
 
     def test_init_api(self):
-
         decorator = TraceDecorator(
-            target=self.target_api, full_thread=self.full_thread_api,
+            target=self.target_api,
+            full_thread=self.full_thread_api,
         )
         assert decorator.target == self.target_api
         assert decorator.full_thread == self.full_thread_api
 
     def test_init_message(self):
-
         x_msg_id = self.msg_id
         x_thread_id = self.thread_id
         x_trace_report = TraceReport(
@@ -45,7 +43,9 @@ class TestTraceDecorator(TestCase):
         decorator = TraceDecorator(
             target=self.target_msg,
             full_thread=self.full_thread_msg,
-            trace_reports=[x_trace_report,],
+            trace_reports=[
+                x_trace_report,
+            ],
         )
         assert decorator.target == self.target_msg
         assert decorator.full_thread == self.full_thread_msg
@@ -61,7 +61,6 @@ class TestTraceDecorator(TestCase):
         assert trace_report.outcome == self.outcome
 
     def test_serialize_load(self):
-
         x_msg_id = self.msg_id
         x_thread_id = self.thread_id
         x_trace_report = TraceReport(
@@ -78,7 +77,10 @@ class TestTraceDecorator(TestCase):
         decorator = TraceDecorator(
             target=self.target_msg,
             full_thread=self.full_thread_msg,
-            trace_reports=[x_trace_report, x_trace_report,],
+            trace_reports=[
+                x_trace_report,
+                x_trace_report,
+            ],
         )
 
         dumped = decorator.serialize()
@@ -99,7 +101,8 @@ class TestTraceDecorator(TestCase):
 
     def test_trace_reports(self):
         decorator = TraceDecorator(
-            target=self.target_msg, full_thread=self.full_thread_msg,
+            target=self.target_msg,
+            full_thread=self.full_thread_msg,
         )
         assert len(decorator.trace_reports) == 0
 
