@@ -1,39 +1,70 @@
 """Classes for BaseStorage-based record management."""
 
 import json
+import logging
 import sys
 import uuid
 
 from datetime import datetime
-from typing import Any, Mapping, Sequence, Union
+from typing import Any, Mapping, Optional, Sequence, Type, TypeVar, Union
 
 from marshmallow import fields
 
 from ...cache.base import BaseCache
-from ...config.injection_context import InjectionContext
+from ...config.settings import BaseSettings
+from ...core.profile import ProfileSession
 from ...storage.base import BaseStorage, StorageDuplicateError, StorageNotFoundError
 from ...storage.record import StorageRecord
 
-from .base import BaseModel, BaseModelSchema
-from ..responder import BaseResponder
 from ..util import datetime_to_str, time_now
 from ..valid import INDY_ISO8601_DATETIME
 
+from .base import BaseModel, BaseModelSchema, BaseModelError
 
-def match_post_filter(record: dict, post_filter: dict, positive: bool = True) -> bool:
-    """Determine if a record value matches the post-filter.
+LOGGER = logging.getLogger(__name__)
+
+
+RecordType = TypeVar("RecordType", bound="BaseRecord")
+
+
+def match_post_filter(
+    record: dict,
+    post_filter: dict,
+    positive: bool = True,
+    alt: bool = False,
+) -> bool:
+    """
+    Determine if a record value matches the post-filter.
 
     Args:
         record: record to check
         post_filter: filter to apply (empty or None filter matches everything)
         positive: whether matching all filter criteria positively or negatively
+        alt: set to match any (positive=True) value or miss all (positive=False)
+            values in post_filter
     """
     if not post_filter:
         return True
 
+    if alt:
+        return (
+            positive
+            and all(
+                record.get(k) and record.get(k) in alts
+                for k, alts in post_filter.items()
+            )
+        ) or (
+            (not positive)
+            and all(
+                record.get(k) and record.get(k) not in alts
+                for k, alts in post_filter.items()
+            )
+        )
+
     for k, v in post_filter.items():
         if record.get(k) != v:
             return not positive
+
     return positive
 
 
@@ -43,13 +74,14 @@ class BaseRecord(BaseModel):
     class Meta:
         """BaseRecord metadata."""
 
+    DEFAULT_CACHE_TTL = 60
     RECORD_ID_NAME = "id"
     RECORD_TYPE = None
-    WEBHOOK_TOPIC = None
+    RECORD_TOPIC: Optional[str] = None
+    EVENT_NAMESPACE: str = "acapy::record"
     LOG_STATE_FLAG = None
-    CACHE_TTL = 60
-    CACHE_ENABLED = False
     TAG_NAMES = {"state"}
+    STATE_DELETED = "deleted"
 
     def __init__(
         self,
@@ -58,6 +90,7 @@ class BaseRecord(BaseModel):
         *,
         created_at: Union[str, datetime] = None,
         updated_at: Union[str, datetime] = None,
+        new_with_id: bool = False,
     ):
         """Initialize a new BaseRecord."""
         if not self.RECORD_TYPE:
@@ -68,13 +101,15 @@ class BaseRecord(BaseModel):
             )
         self._id = id
         self._last_state = state
+        self._new_with_id = new_with_id
         self.state = state
         self.created_at = datetime_to_str(created_at)
         self.updated_at = datetime_to_str(updated_at)
 
     @classmethod
     def from_storage(cls, record_id: str, record: Mapping[str, Any]):
-        """Initialize a record from its stored representation.
+        """
+        Initialize a record from its stored representation.
 
         Args:
             record_id: The unique record identifier
@@ -82,7 +117,7 @@ class BaseRecord(BaseModel):
         """
         record_id_name = cls.RECORD_ID_NAME
         if record_id_name in record:
-            raise ValueError(f"Duplicate {record_id_name} inputs")
+            raise ValueError(f"Duplicate {record_id_name} inputs; {record}")
         params = dict(**record)
         params[record_id_name] = record_id
         return cls(**params)
@@ -90,11 +125,13 @@ class BaseRecord(BaseModel):
     @classmethod
     def get_tag_map(cls) -> Mapping[str, str]:
         """Accessor for the set of defined tags."""
+
         return {tag.lstrip("~"): tag for tag in cls.TAG_NAMES or ()}
 
     @property
     def storage_record(self) -> StorageRecord:
         """Accessor for a `StorageRecord` representing this record."""
+
         return StorageRecord(
             self.RECORD_TYPE, json.dumps(self.value), self.tags, self._id
         )
@@ -102,11 +139,13 @@ class BaseRecord(BaseModel):
     @property
     def record_value(self) -> dict:
         """Accessor to define custom properties for the JSON record value."""
+
         return {}
 
     @property
     def value(self) -> dict:
         """Accessor for the JSON record value generated for this record."""
+
         ret = self.strip_tag_prefix(self.tags)
         ret.update({"created_at": self.created_at, "updated_at": self.updated_at})
         ret.update(self.record_value)
@@ -115,6 +154,7 @@ class BaseRecord(BaseModel):
     @property
     def record_tags(self) -> dict:
         """Accessor to define implementation-specific tags."""
+
         return {
             tag: getattr(self, prop)
             for (prop, tag) in self.get_tag_map().items()
@@ -124,122 +164,113 @@ class BaseRecord(BaseModel):
     @property
     def tags(self) -> dict:
         """Accessor for the record tags generated for this record."""
+
         tags = self.record_tags
         return tags
 
     @classmethod
-    def cache_key(cls, record_id: str, record_type: str = None):
-        """Assemble a cache key.
-
-        Args:
-            record_id: The record identifier
-            record_type The cache type identifier, defaulting to RECORD_TYPE
+    async def get_cached_key(cls, session: ProfileSession, cache_key: str):
         """
-        if not record_type:
-            record_type = cls.RECORD_TYPE
-        if record_id:
-            return f"{record_type}::{record_id}"
-
-    @classmethod
-    async def get_cached_key(cls, context: InjectionContext, cache_key: str):
-        """Shortcut method to fetch a cached key value.
+        Shortcut method to fetch a cached key value.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             cache_key: The unique cache identifier
         """
         if not cache_key:
             return
-        cache: BaseCache = await context.inject(BaseCache, required=False)
+        cache = session.inject_or(BaseCache)
         if cache:
             return await cache.get(cache_key)
 
     @classmethod
     async def set_cached_key(
-        cls, context: InjectionContext, cache_key: str, value: Any, ttl=None
+        cls, session: ProfileSession, cache_key: str, value: Any, ttl=None
     ):
-        """Shortcut method to set a cached key value.
+        """
+        Shortcut method to set a cached key value.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             cache_key: The unique cache identifier
             value: The value to cache
             ttl: The cache ttl
         """
+
         if not cache_key:
             return
-        cache: BaseCache = await context.inject(BaseCache, required=False)
+        cache = session.inject_or(BaseCache)
         if cache:
-            await cache.set(cache_key, value, ttl or cls.CACHE_TTL)
+            await cache.set(cache_key, value, ttl or cls.DEFAULT_CACHE_TTL)
 
     @classmethod
-    async def clear_cached_key(cls, context: InjectionContext, cache_key: str):
-        """Shortcut method to clear a cached key value, if any.
+    async def clear_cached_key(cls, session: ProfileSession, cache_key: str):
+        """
+        Shortcut method to clear a cached key value, if any.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             cache_key: The unique cache identifier
         """
+
         if not cache_key:
             return
-        cache: BaseCache = await context.inject(BaseCache, required=False)
+        cache = session.inject_or(BaseCache)
         if cache:
             await cache.clear(cache_key)
 
-    async def clear_cached(self, context: InjectionContext):
-        """Clear the cached value of this record, if any."""
-        await self.clear_cached_key(context, self.cache_key(self._id))
-
     @classmethod
     async def retrieve_by_id(
-        cls, context: InjectionContext, record_id: str, cached: bool = True
-    ) -> "BaseRecord":
-        """Retrieve a stored record by ID.
+        cls: Type[RecordType],
+        session: ProfileSession,
+        record_id: str,
+        *,
+        for_update=False,
+    ) -> RecordType:
+        """
+        Retrieve a stored record by ID.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             record_id: The ID of the record to find
-            cached: Whether to check the cache for this record
         """
-        cache_key = cls.cache_key(record_id)
-        vals = None
-
-        if cls.CACHE_ENABLED and cached:
-            vals = await cls.get_cached_key(context, cache_key)
-
-        if not vals:
-            storage: BaseStorage = await context.inject(BaseStorage)
-            result = await storage.get_record(
-                cls.RECORD_TYPE, record_id, {"retrieveTags": False}
-            )
-            vals = json.loads(result.value)
-            if cls.CACHE_ENABLED:
-                await cls.set_cached_key(context, cache_key, vals)
 
+        storage = session.inject(BaseStorage)
+        result = await storage.get_record(
+            cls.RECORD_TYPE, record_id, {"forUpdate": for_update, "retrieveTags": False}
+        )
+        vals = json.loads(result.value)
         return cls.from_storage(record_id, vals)
 
     @classmethod
     async def retrieve_by_tag_filter(
-        cls, context: InjectionContext, tag_filter: dict, post_filter: dict = None
-    ) -> "BaseRecord":
-        """Retrieve a record by tag filter.
+        cls: Type[RecordType],
+        session: ProfileSession,
+        tag_filter: dict,
+        post_filter: dict = None,
+        *,
+        for_update=False,
+    ) -> RecordType:
+        """
+        Retrieve a record by tag filter.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             tag_filter: The filter dictionary to apply
-            post_filter: Additional value filters to apply after retrieval
+            post_filter: Additional value filters to apply matching positively,
+                with sequence values specifying alternatives to match (hit any)
         """
-        storage: BaseStorage = await context.inject(BaseStorage)
-        query = storage.search_records(
+
+        storage = session.inject(BaseStorage)
+        rows = await storage.find_all_records(
             cls.RECORD_TYPE,
             cls.prefix_tag_filter(tag_filter),
-            None,
-            {"retrieveTags": False},
+            options={"forUpdate": for_update, "retrieveTags": False},
         )
         found = None
-        async for record in query:
+        for record in rows:
             vals = json.loads(record.value)
-            if match_post_filter(vals, post_filter):
+            if match_post_filter(vals, post_filter, alt=False):
                 if found:
                     raise StorageDuplicateError(
                         "Multiple {} records located for {}{}".format(
@@ -259,158 +290,175 @@ class BaseRecord(BaseModel):
 
     @classmethod
     async def query(
-        cls,
-        context: InjectionContext,
+        cls: Type[RecordType],
+        session: ProfileSession,
         tag_filter: dict = None,
+        *,
         post_filter_positive: dict = None,
         post_filter_negative: dict = None,
-    ) -> Sequence["BaseRecord"]:
-        """Query stored records.
+        alt: bool = False,
+    ) -> Sequence[RecordType]:
+        """
+        Query stored records.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             tag_filter: An optional dictionary of tag filter clauses
             post_filter_positive: Additional value filters to apply matching positively
             post_filter_negative: Additional value filters to apply matching negatively
+            alt: set to match any (positive=True) value or miss all (positive=False)
+                values in post_filter
         """
-        storage: BaseStorage = await context.inject(BaseStorage)
-        query = storage.search_records(
+
+        storage = session.inject(BaseStorage)
+        rows = await storage.find_all_records(
             cls.RECORD_TYPE,
             cls.prefix_tag_filter(tag_filter),
-            None,
-            {"retrieveTags": False},
+            options={"retrieveTags": False},
         )
         result = []
-        async for record in query:
+        for record in rows:
             vals = json.loads(record.value)
             if match_post_filter(
-                vals, post_filter_positive, True
-            ) and match_post_filter(vals, post_filter_negative, False):
-                result.append(cls.from_storage(record.id, vals))
+                vals,
+                post_filter_positive,
+                positive=True,
+                alt=alt,
+            ) and match_post_filter(
+                vals,
+                post_filter_negative,
+                positive=False,
+                alt=alt,
+            ):
+                try:
+                    result.append(cls.from_storage(record.id, vals))
+                except BaseModelError as err:
+                    raise BaseModelError(f"{err}, for record id {record.id}")
         return result
 
     async def save(
         self,
-        context: InjectionContext,
+        session: ProfileSession,
         *,
         reason: str = None,
         log_params: Mapping[str, Any] = None,
         log_override: bool = False,
-        webhook: bool = None,
+        event: bool = None,
     ) -> str:
-        """Persist the record to storage.
+        """
+        Persist the record to storage.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             reason: A reason to add to the log
             log_params: Additional parameters to log
-            webhook: Flag to override whether the webhook is sent
+            override: Override configured logging regimen, print to stderr instead
+            event: Flag to override whether the event is sent
         """
+
         new_record = None
         log_reason = reason or ("Updated record" if self._id else "Created record")
         try:
             self.updated_at = time_now()
-            storage: BaseStorage = await context.inject(BaseStorage)
-            if self._id:
+            storage = session.inject(BaseStorage)
+            if self._id and not self._new_with_id:
                 record = self.storage_record
-                await storage.update_record_value(record, record.value)
-                await storage.update_record_tags(record, record.tags)
+                await storage.update_record(record, record.value, record.tags)
                 new_record = False
             else:
-                self._id = str(uuid.uuid4())
+                if not self._id:
+                    self._id = str(uuid.uuid4())
                 self.created_at = self.updated_at
                 await storage.add_record(self.storage_record)
                 new_record = True
+                self._new_with_id = False
         finally:
             params = {self.RECORD_TYPE: self.serialize()}
             if log_params:
                 params.update(log_params)
             if new_record is None:
                 log_reason = f"FAILED: {log_reason}"
-            self.log_state(context, log_reason, params, override=log_override)
+            self.log_state(
+                log_reason, params, override=log_override, settings=session.settings
+            )
 
-        await self.post_save(context, new_record, self._last_state, webhook)
+        await self.post_save(session, new_record, self._last_state, event)
         self._last_state = self.state
 
         return self._id
 
     async def post_save(
         self,
-        context: InjectionContext,
+        session: ProfileSession,
         new_record: bool,
-        last_state: str,
-        webhook: bool = None,
+        last_state: Optional[str],
+        event: bool = None,
     ):
-        """Perform post-save actions.
+        """
+        Perform post-save actions.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
             new_record: Flag indicating if the record was just created
             last_state: The previous state value
-            webhook: Adjust whether the webhook is called
+            event: Flag to override whether the event is sent
         """
-        await self.clear_cached(context)
 
-        webhook_topic = self.webhook_topic
-        if webhook is None:
-            webhook = bool(webhook_topic) and (new_record or (last_state != self.state))
-        if webhook:
-            await self.send_webhook(
-                context, self.webhook_payload, topic=self.webhook_topic
-            )
+        if event is None:
+            event = new_record or (last_state != self.state)
+        if event:
+            await self.emit_event(session, self.serialize())
 
-    async def delete_record(self, context: InjectionContext):
-        """Remove the stored record.
+    async def delete_record(self, session: ProfileSession):
+        """
+        Remove the stored record.
 
         Args:
-            context: The injection context to use
+            session: The profile session to use
         """
+
         if self._id:
-            storage: BaseStorage = await context.inject(BaseStorage)
+            storage = session.inject(BaseStorage)
+            if self.state:
+                self._previous_state = self.state
+                self.state = BaseRecord.STATE_DELETED
+                await self.emit_event(session, self.serialize())
             await storage.delete_record(self.storage_record)
-        # FIXME - update state and send webhook?
 
-    @property
-    def webhook_payload(self):
-        """Return a JSON-serialized version of the record for the webhook."""
-        return self.serialize()
-
-    @property
-    def webhook_topic(self):
-        """Return the webhook topic value."""
-        return self.WEBHOOK_TOPIC
-
-    async def send_webhook(
-        self, context: InjectionContext, payload: Any, topic: str = None
-    ):
-        """Send a standard webhook.
+    async def emit_event(self, session: ProfileSession, payload: Any = None):
+        """
+        Emit an event.
 
         Args:
-            context: The injection context to use
-            payload: The webhook payload
-            topic: The webhook topic, defaulting to WEBHOOK_TOPIC
+            session: The profile session to use
+            payload: The event payload
         """
-        if not payload:
+
+        if not self.RECORD_TOPIC:
             return
-        if not topic:
-            topic = self.webhook_topic
-            if not topic:
-                return
-        responder: BaseResponder = await context.inject(BaseResponder, required=False)
-        if responder:
-            await responder.send_webhook(topic, payload)
+
+        if self.state:
+            topic = f"{self.EVENT_NAMESPACE}::{self.RECORD_TOPIC}::{self.state}"
+        else:
+            topic = f"{self.EVENT_NAMESPACE}::{self.RECORD_TOPIC}"
+
+        if not payload:
+            payload = self.serialize()
+
+        await session.profile.notify(topic, payload)
 
     @classmethod
     def log_state(
         cls,
-        context: InjectionContext,
         msg: str,
         params: dict = None,
+        settings: BaseSettings = None,
         override: bool = False,
     ):
         """Print a message with increased visibility (for testing)."""
+
         if override or (
-            cls.LOG_STATE_FLAG and context.settings.get(cls.LOG_STATE_FLAG)
+            cls.LOG_STATE_FLAG and settings and settings.get(cls.LOG_STATE_FLAG)
         ):
             out = msg + "\n"
             if params:
@@ -421,6 +469,7 @@ class BaseRecord(BaseModel):
     @classmethod
     def strip_tag_prefix(cls, tags: dict):
         """Strip tilde from unencrypted tag names."""
+
         return (
             {(k[1:] if "~" in k else k): v for (k, v) in tags.items()} if tags else {}
         )
@@ -428,6 +477,7 @@ class BaseRecord(BaseModel):
     @classmethod
     def prefix_tag_filter(cls, tag_filter: dict):
         """Prefix unencrypted tags used in the tag filter."""
+
         ret = None
         if tag_filter:
             tag_map = cls.get_tag_map()
@@ -443,10 +493,29 @@ class BaseRecord(BaseModel):
 
     def __eq__(self, other: Any) -> bool:
         """Comparison between records."""
+
         if type(other) is type(self):
             return self.value == other.value and self.tags == other.tags
         return False
 
+    @classmethod
+    def get_attributes_by_prefix(cls, prefix: str, walk_mro: bool = True):
+        """
+        List all values for attributes with common prefix.
+
+        Args:
+            prefix: Common prefix to look for
+            walk_mro: Walk MRO to find attributes inherited from superclasses
+        """
+
+        bases = cls.__mro__ if walk_mro else [cls]
+        return [
+            vars(base)[name]
+            for base in bases
+            for name in vars(base)
+            if name.startswith(prefix)
+        ]
+
 
 class BaseExchangeRecord(BaseRecord):
     """Represents a base record with event tracing capability."""
@@ -459,12 +528,14 @@ class BaseExchangeRecord(BaseRecord):
         trace: bool = False,
         **kwargs,
     ):
-        """Initialize a new V10CredentialExchange."""
+        """Initialize a new BaseExchangeRecord."""
+
         super().__init__(id, state, **kwargs)
         self.trace = trace
 
     def __eq__(self, other: Any) -> bool:
         """Comparison between records."""
+
         if type(other) is type(self):
             return (
                 self.value == other.value
@@ -483,7 +554,9 @@ class BaseRecordSchema(BaseModelSchema):
         model_class = None
 
     state = fields.Str(
-        required=False, description="Current record state", example="active"
+        required=False,
+        description="Current record state",
+        example="active",
     )
     created_at = fields.Str(
         required=False, description="Time of record creation", **INDY_ISO8601_DATETIME
