@@ -9,11 +9,13 @@ from .....problem_report.v1_0.message import ProblemReport
 
 from ...handlers.invitation_handler import InvitationHandler
 from ...messages.problem_report_reason import ProblemReportReason
+from ......wallet.did_method import DIDMethods
 
 
 @pytest.fixture()
 def request_context() -> RequestContext:
     ctx = RequestContext.test_context()
+    ctx.injector.bind_instance(DIDMethods, DIDMethods())
     ctx.message_receipt = MessageReceipt()
     yield ctx
 
