@@ -1,4 +1,4 @@
-"""Feature discovery v2 admin routes."""
+"""Feature discovery admin routes."""
 
 from aiohttp import web
 from aiohttp_apispec import docs, querystring_schema, response_schema
@@ -10,30 +10,21 @@ from ....messaging.models.openapi import OpenAPISchema
 from ....messaging.valid import UUIDFour
 from ....storage.error import StorageNotFoundError, StorageError
 
-from .manager import V20DiscoveryMgr
+from .manager import V10DiscoveryMgr
 from .message_types import SPEC_URI
 from .models.discovery_record import (
-    V20DiscoveryRecordSchema,
-    V20DiscoveryExchangeRecord,
+    V10DiscoveryExchangeRecord,
+    V10DiscoveryRecordSchema,
 )
 
 
-class V20DiscoveryExchangeResultSchema(OpenAPISchema):
-    """Result schema for Discover Features v2.0 exchange record."""
-
-    results = fields.Nested(
-        V20DiscoveryRecordSchema,
-        description="Discover Features v2.0 exchange record",
-    )
-
-
-class V20DiscoveryExchangeListResultSchema(OpenAPISchema):
-    """Result schema for Discover Features v2.0 exchange records."""
+class V10DiscoveryExchangeListResultSchema(OpenAPISchema):
+    """Result schema for Discover Features v1.0 exchange records."""
 
     results = fields.List(
         fields.Nested(
-            V20DiscoveryRecordSchema,
-            description="Discover Features v2.0 exchange record",
+            V10DiscoveryRecordSchema,
+            description="Discover Features v1.0 exchange record",
         )
     )
 
@@ -41,12 +32,10 @@ class V20DiscoveryExchangeListResultSchema(OpenAPISchema):
 class QueryFeaturesQueryStringSchema(OpenAPISchema):
     """Query string parameters for feature query."""
 
-    query_protocol = fields.Str(
-        description="Protocol feature-type query", required=False, example="*"
-    )
-    query_goal_code = fields.Str(
-        description="Goal-code feature-type query", required=False, example="*"
+    query = fields.Str(
+        description="Protocol feature query", required=False, example="*"
     )
+    comment = fields.Str(description="Comment", required=False, example="test")
     connection_id = fields.Str(
         description=(
             "Connection identifier, if none specified, "
@@ -58,7 +47,7 @@ class QueryFeaturesQueryStringSchema(OpenAPISchema):
 
 
 class QueryDiscoveryExchRecordsSchema(OpenAPISchema):
-    """Query string parameter for Discover Features v2.0 exchange record."""
+    """Query string parameter for Discover Features v1.0 exchange record."""
 
     connection_id = fields.Str(
         description="Connection identifier",
@@ -68,51 +57,51 @@ class QueryDiscoveryExchRecordsSchema(OpenAPISchema):
 
 
 @docs(
-    tags=["discover-features v2.0"],
+    tags=["discover-features"],
     summary="Query supported features",
 )
 @querystring_schema(QueryFeaturesQueryStringSchema())
-@response_schema(V20DiscoveryExchangeResultSchema(), 200, description="")
+@response_schema(V10DiscoveryRecordSchema(), 200, description="")
 async def query_features(request: web.BaseRequest):
     """
-    Request handler for creating and sending feature queries.
+    Request handler for creating and sending feature query.
 
     Args:
         request: aiohttp request object
 
     Returns:
-        V20DiscoveryExchangeRecord
+        V10DiscoveryExchangeRecord
 
     """
     context: AdminRequestContext = request["context"]
     profile = context.profile
-    mgr = V20DiscoveryMgr(profile)
-    query_protocol = request.query.get("query_protocol", "*")
-    query_goal_code = request.query.get("query_goal_code", "*")
+    mgr = V10DiscoveryMgr(profile)
+    query = request.query.get("query", "*")
+    comment = request.query.get("comment", "*")
     connection_id = request.query.get("connection_id")
     result = await mgr.create_and_send_query(
         connection_id=connection_id,
-        query_protocol=query_protocol,
-        query_goal_code=query_goal_code,
+        query=query,
+        comment=comment,
     )
     return web.json_response(result.serialize())
 
 
 @docs(
-    tags=["discover-features v2.0"],
-    summary="Discover Features v2.0 records",
+    tags=["discover-features"],
+    summary="Discover Features records",
 )
 @querystring_schema(QueryDiscoveryExchRecordsSchema())
-@response_schema(V20DiscoveryExchangeListResultSchema(), 200, description="")
+@response_schema(V10DiscoveryExchangeListResultSchema(), 200, description="")
 async def query_records(request: web.BaseRequest):
     """
-    Request handler for looking up V20DiscoveryExchangeRecord.
+    Request handler for looking up V10DiscoveryExchangeRecord.
 
     Args:
         request: aiohttp request object
 
     Returns:
-        List of V20DiscoveryExchangeRecord
+        List of V10DiscoveryExchangeRecord
 
     """
     context: AdminRequestContext = request["context"]
@@ -120,14 +109,14 @@ async def query_records(request: web.BaseRequest):
     if not connection_id:
         try:
             async with context.profile.session() as session:
-                records = await V20DiscoveryExchangeRecord.query(session=session)
+                records = await V10DiscoveryExchangeRecord.query(session=session)
             results = [record.serialize() for record in records]
         except (StorageError, BaseModelError) as err:
             raise web.HTTPBadRequest(reason=err.roll_up) from err
     else:
         try:
             async with context.profile.session() as session:
-                record = await V20DiscoveryExchangeRecord.retrieve_by_connection_id(
+                record = await V10DiscoveryExchangeRecord.retrieve_by_connection_id(
                     session=session, connection_id=connection_id
                 )
             # There should only be one record for a connection
@@ -142,8 +131,8 @@ async def register(app: web.Application):
 
     app.add_routes(
         [
-            web.get("/discover-features-2.0/queries", query_features, allow_head=False),
-            web.get("/discover-features-2.0/records", query_records, allow_head=False),
+            web.get("/discover-features/query", query_features, allow_head=False),
+            web.get("/discover-features/records", query_records, allow_head=False),
         ]
     )
 
@@ -156,8 +145,8 @@ def post_process_routes(app: web.Application):
         app._state["swagger_dict"]["tags"] = []
     app._state["swagger_dict"]["tags"].append(
         {
-            "name": "discover-features v2.0",
-            "description": "Feature discovery v2",
+            "name": "discover-features",
+            "description": "Feature discovery",
             "externalDocs": {"description": "Specification", "url": SPEC_URI},
         }
     )
