@@ -9,11 +9,12 @@ from ......indy.models.pres_preview import (
 from ......messaging.decorators.attach_decorator import AttachDecorator
 from ......messaging.models.base_record import BaseExchangeRecord, BaseExchangeSchema
 
-from ...message_types import PRESENTATION_PROPOSAL
-from ...messages.presentation_proposal import PresentationProposal
+from ...message_types import ATTACHMENT_FORMAT, PRES_20_PROPOSAL
+from ...messages.pres_format import V20PresFormat
+from ...messages.pres_proposal import V20PresProposal
 
-from .. import presentation_exchange as test_module
-from ..presentation_exchange import V10PresentationExchange
+from .. import pres_exchange as test_module
+from ..pres_exchange import V20PresExRecord
 
 S_ID = "NcYxiDXkpYi6ov5FcYDi1e:2:vidya:1.0"
 CD_ID = f"NcYxiDXkpYi6ov5FcYDi1e:3:CL:{S_ID}:tag1"
@@ -57,22 +58,6 @@ INDY_PROOF_REQ = {
         }
     },
 }
-PRES_PREVIEW = IndyPresPreview(
-    attributes=[
-        IndyPresAttrSpec(name="player", cred_def_id=CD_ID, value="Richie Knucklez"),
-        IndyPresAttrSpec(
-            name="screenCapture",
-            cred_def_id=CD_ID,
-            mime_type="image/png",
-            value="aW1hZ2luZSBhIHNjcmVlbiBjYXB0dXJl",
-        ),
-    ],
-    predicates=[
-        IndyPresPredSpec(
-            name="highScore", cred_def_id=CD_ID, predicate=">=", threshold=1000000
-        )
-    ],
-)
 
 
 class BasexRecordImpl(BaseExchangeRecord):
@@ -89,31 +74,46 @@ class BasexRecordImplSchema(BaseExchangeSchema):
 
 class TestRecord(AsyncTestCase):
     async def test_record(self):
-        presentation_proposal = PresentationProposal(
-            comment="Hello World", presentation_proposal=PRES_PREVIEW
+        pres_proposal = V20PresProposal(
+            comment="Hello World",
+            formats=[
+                V20PresFormat(
+                    attach_id="indy",
+                    format_=ATTACHMENT_FORMAT[PRES_20_PROPOSAL][
+                        V20PresFormat.Format.INDY.api
+                    ],
+                )
+            ],
+            proposals_attach=[
+                AttachDecorator.data_base64(INDY_PROOF_REQ, ident="indy")
+            ],
         )
-        record = V10PresentationExchange(
-            presentation_exchange_id="pxid",
-            connection_id="conn_id",
+        record = V20PresExRecord(
+            pres_ex_id="pxid",
             thread_id="thid",
+            connection_id="conn_id",
+            initiator="init",
+            role="role",
+            state="state",
+            verified="false",
             auto_present=True,
+            error_msg="error",
         )
-        record.presentation_proposal_dict = presentation_proposal  # cover setter
-        record.presentation_request_dict = None  # cover setter
+        record.pres_proposal = pres_proposal  # cover setter
 
-        assert record.presentation_exchange_id == "pxid"
+        assert record.pres_ex_id == "pxid"
 
         assert record.record_value == {
             "connection_id": "conn_id",
-            "initiator": None,
-            "presentation_proposal_dict": presentation_proposal.serialize(),
-            "role": None,
-            "state": None,
+            "initiator": "init",
+            "role": "role",
+            "state": "state",
+            "pres_proposal": pres_proposal.serialize(),
+            "verified": "false",
+            "verified_msgs": None,
             "auto_present": True,
             "auto_verify": False,
-            "error_msg": None,
-            "verified": None,
-            "verified_msgs": None,
+            "error_msg": "error",
             "trace": False,
         }
 
@@ -122,11 +122,11 @@ class TestRecord(AsyncTestCase):
 
     async def test_save_error_state(self):
         session = InMemoryProfile.test_session()
-        record = V10PresentationExchange(state=None)
+        record = V20PresExRecord(state=None)
         assert record._last_state is None
         await record.save_error_state(session)  # cover short circuit
 
-        record.state = V10PresentationExchange.STATE_PROPOSAL_RECEIVED
+        record.state = V20PresExRecord.STATE_PROPOSAL_RECEIVED
         await record.save(session)
 
         with async_mock.patch.object(
