@@ -4,7 +4,7 @@ import asyncio
 from abc import ABC, abstractmethod
 from typing import Union
 
-from ...config.injection_context import InjectionContext
+from ...core.profile import Profile
 from ...utils.stats import Collector
 
 from ..error import TransportError
@@ -14,10 +14,15 @@ from ..wire_format import BaseWireFormat
 class BaseOutboundTransport(ABC):
     """Base outbound transport class."""
 
-    def __init__(self, wire_format: BaseWireFormat = None) -> None:
+    def __init__(
+        self,
+        wire_format: BaseWireFormat = None,
+        root_profile: Profile = None,
+    ) -> None:
         """Initialize a `BaseOutboundTransport` instance."""
         self._collector = None
         self._wire_format = wire_format
+        self.root_profile = root_profile
 
     @property
     def collector(self) -> Collector:
@@ -29,6 +34,16 @@ class BaseOutboundTransport(ABC):
         """Assign a new stats collector instance."""
         self._collector = coll
 
+    @property
+    def wire_format(self) -> BaseWireFormat:
+        """Accessor for a custom wire format for the transport."""
+        return self._wire_format
+
+    @wire_format.setter
+    def wire_format(self, format: BaseWireFormat):
+        """Setter for a custom wire format for the transport."""
+        self._wire_format = format
+
     async def __aenter__(self):
         """Async context manager enter."""
         await self.start()
@@ -47,27 +62,22 @@ class BaseOutboundTransport(ABC):
     async def stop(self):
         """Shut down the transport."""
 
-    @property
-    def wire_format(self) -> BaseWireFormat:
-        """Accessor for a custom wire format for the transport."""
-        return self._wire_format
-
-    @wire_format.setter
-    def wire_format(self, format: BaseWireFormat):
-        """Setter for a custom wire format for the transport."""
-        self._wire_format = format
-
     @abstractmethod
     async def handle_message(
-        self, context: InjectionContext, payload: Union[str, bytes], endpoint: str
+        self,
+        profile: Profile,
+        payload: Union[str, bytes],
+        endpoint: str,
+        metadata: dict = None,
     ):
         """
-        Handle message from queue.
+        Handle message.
 
         Args:
-            context: the context that produced the message
+            profile: the profile that produced the message
             payload: message payload in string or byte format
             endpoint: URI endpoint for delivery
+            metadata: Additional metadata associated with the payload
         """
 
 
