@@ -9,7 +9,7 @@ from typing import Callable, Type, Union
 from urllib.parse import urlparse
 
 from ...connections.models.connection_target import ConnectionTarget
-from ...config.injection_context import InjectionContext
+from ...core.profile import Profile
 from ...utils.classloader import ClassLoader, ModuleLoadError, ClassNotFoundError
 from ...utils.stats import Collector
 from ...utils.task_queue import CompletedTask, TaskQueue, task_exc_info
@@ -41,13 +41,13 @@ class QueuedOutboundMessage:
 
     def __init__(
         self,
-        context: InjectionContext,
+        profile: Profile,
         message: OutboundMessage,
         target: ConnectionTarget,
         transport_id: str,
     ):
         """Initialize the queued outbound message."""
-        self.context = context
+        self.profile = profile
         self.endpoint = target and target.endpoint
         self.error: Exception = None
         self.message = message
@@ -58,6 +58,8 @@ class QueuedOutboundMessage:
         self.target = target
         self.task: asyncio.Task = None
         self.transport_id: str = transport_id
+        self.metadata: dict = None
+        self.api_key: str = None
 
 
 class OutboundTransportManager:
@@ -65,18 +67,16 @@ class OutboundTransportManager:
 
     MAX_RETRY_COUNT = 4
 
-    def __init__(
-        self, context: InjectionContext, handle_not_delivered: Callable = None
-    ):
+    def __init__(self, profile: Profile, handle_not_delivered: Callable = None):
         """
         Initialize a `OutboundTransportManager` instance.
 
         Args:
-            context: The application context
+            root_profile: The application root profile
             handle_not_delivered: An optional handler for undelivered messages
 
         """
-        self.context = context
+        self.root_profile = profile
         self.loop = asyncio.get_event_loop()
         self.handle_not_delivered = handle_not_delivered
         self.outbound_buffer = []
@@ -87,23 +87,25 @@ class OutboundTransportManager:
         self.running_transports = {}
         self.task_queue = TaskQueue(max_active=200)
         self._process_task: asyncio.Task = None
-        if self.context.settings.get("transport.max_outbound_retry"):
-            self.MAX_RETRY_COUNT = self.context.settings["transport.max_outbound_retry"]
+        if self.root_profile.settings.get("transport.max_outbound_retry"):
+            self.MAX_RETRY_COUNT = self.root_profile.settings[
+                "transport.max_outbound_retry"
+            ]
 
     async def setup(self):
         """Perform setup operations."""
         outbound_transports = (
-            self.context.settings.get("transport.outbound_configs") or []
+            self.root_profile.settings.get("transport.outbound_configs") or []
         )
         for outbound_transport in outbound_transports:
             self.register(outbound_transport)
 
-    def register(self, module: str) -> str:
+    def register(self, module_name: str) -> str:
         """
         Register a new outbound transport by module path.
 
         Args:
-            module: Module name to register
+            module_name: Module name to register
 
         Raises:
             OutboundTransportRegistrationError: If the imported class cannot
@@ -115,13 +117,19 @@ class OutboundTransportManager:
 
         """
         try:
+            if "." in module_name:
+                package, module = module_name.split(".", 1)
+            else:
+                package = MODULE_BASE_PATH
+                module = module_name
+
             imported_class = ClassLoader.load_subclass_of(
-                BaseOutboundTransport, module, MODULE_BASE_PATH
+                BaseOutboundTransport, module, package
             )
-        except (ModuleLoadError, ClassNotFoundError):
+        except (ModuleLoadError, ClassNotFoundError) as e:
             raise OutboundTransportRegistrationError(
                 f"Outbound transport module {module} could not be resolved."
-            )
+            ) from e
 
         return self.register_class(imported_class)
 
@@ -169,8 +177,10 @@ class OutboundTransportManager:
 
     async def start_transport(self, transport_id: str):
         """Start a registered transport."""
-        transport = self.registered_transports[transport_id]()
-        transport.collector = await self.context.inject(Collector, required=False)
+        transport = self.registered_transports[transport_id](
+            root_profile=self.root_profile
+        )
+        transport.collector = self.root_profile.inject_or(Collector)
         await transport.start()
         self.running_transports[transport_id] = transport
 
@@ -210,6 +220,17 @@ class OutboundTransportManager:
         except StopIteration:
             pass
 
+    def get_external_running_transport(self) -> str:
+        """Find the external running transport ID."""
+        try:
+            return next(
+                transport_id
+                for transport_id, transport in self.running_transports.items()
+                if transport.is_external
+            )
+        except StopIteration:
+            pass
+
     def get_running_transport_for_endpoint(self, endpoint: str):
         """Find the running transport ID to use for a given endpoint."""
         # Grab the scheme from the uri
@@ -231,12 +252,12 @@ class OutboundTransportManager:
         """Get an instance of a running transport by ID."""
         return self.running_transports[transport_id]
 
-    def enqueue_message(self, context: InjectionContext, outbound: OutboundMessage):
+    async def enqueue_message(self, profile: Profile, outbound: OutboundMessage):
         """
         Add an outbound message to the queue.
 
         Args:
-            context: The context of the request
+            profile: The active profile for the request
             outbound: The outbound message to deliver
         """
         targets = [outbound.target] if outbound.target else (outbound.target_list or [])
@@ -244,21 +265,57 @@ class OutboundTransportManager:
         for target in targets:
             endpoint = target.endpoint
             try:
-                transport_id = self.get_running_transport_for_endpoint(endpoint)
+                transport_id = self.get_external_running_transport()
+                if not transport_id:
+                    transport_id = self.get_running_transport_for_endpoint(endpoint)
             except OutboundDeliveryError:
                 pass
             if transport_id:
                 break
         if not transport_id:
             raise OutboundDeliveryError("No supported transport for outbound message")
+        transport = self.get_transport_instance(transport_id)
+        if transport.is_external:
+            encoded_outbound_message = await self.encode_outbound_message(
+                profile, outbound, target
+            )
+            await transport.handle_message(
+                profile, encoded_outbound_message.payload, target.endpoint
+            )
+        else:
+            queued = QueuedOutboundMessage(profile, outbound, target, transport_id)
+            queued.retries = self.MAX_RETRY_COUNT
+            self.outbound_new.append(queued)
+            self.process_queued()
 
-        queued = QueuedOutboundMessage(context, outbound, target, transport_id)
-        queued.retries = self.MAX_RETRY_COUNT
-        self.outbound_new.append(queued)
-        self.process_queued()
+    async def encode_outbound_message(
+        self, profile: Profile, outbound: OutboundMessage, target: ConnectionTarget
+    ):
+        """
+        Encode outbound message for the target.
+
+        Args:
+            profile: The active profile for the request
+            outbound: The outbound message to deliver
+            target: The outbound message target
+        """
+
+        outbound_message = QueuedOutboundMessage(profile, outbound, target, None)
+
+        if outbound_message.message.enc_payload:
+            outbound_message.payload = outbound_message.message.enc_payload
+        else:
+            await self.perform_encode(outbound_message)
+
+        return outbound_message
 
     def enqueue_webhook(
-        self, topic: str, payload: dict, endpoint: str, max_attempts: int = None
+        self,
+        topic: str,
+        payload: dict,
+        endpoint: str,
+        max_attempts: int = None,
+        metadata: dict = None,
     ):
         """
         Add a webhook to the queue.
@@ -268,6 +325,7 @@ class OutboundTransportManager:
             payload: The webhook payload
             endpoint: The webhook endpoint
             max_attempts: Override the maximum number of attempts
+            metadata: Additional metadata associated with the payload
 
         Raises:
             OutboundDeliveryError: if the associated transport is not running
@@ -275,7 +333,13 @@ class OutboundTransportManager:
         """
         transport_id = self.get_running_transport_for_endpoint(endpoint)
         queued = QueuedOutboundMessage(None, None, None, transport_id)
+        if len(endpoint.split("#")) > 1:
+            endpoint_hash_split = endpoint.split("#")
+            endpoint = endpoint_hash_split[0]
+            api_key = endpoint_hash_split[1]
+            queued.api_key = api_key
         queued.endpoint = f"{endpoint}/topic/{topic}/"
+        queued.metadata = metadata
         queued.payload = json.dumps(payload)
         queued.state = QueuedOutboundMessage.STATE_PENDING
         queued.retries = 4 if max_attempts is None else max_attempts - 1
@@ -325,8 +389,8 @@ class OutboundTransportManager:
                             queued.endpoint,
                             exc_info=queued.error,
                         )
-                        if self.handle_not_delivered:
-                            self.handle_not_delivered(queued.context, queued.message)
+                        if self.handle_not_delivered and queued.message:
+                            self.handle_not_delivered(queued.profile, queued.message)
                     continue  # remove from buffer
 
                 deliver = False
@@ -343,14 +407,14 @@ class OutboundTransportManager:
                 if deliver:
                     queued.state = QueuedOutboundMessage.STATE_DELIVER
                     p_time = trace_event(
-                        self.context.settings,
+                        self.root_profile.settings,
                         queued.message if queued.message else queued.payload,
                         outcome="OutboundTransportManager.DELIVER.START."
                         + queued.endpoint,
                     )
                     self.deliver_queued_message(queued)
                     trace_event(
-                        self.context.settings,
+                        self.root_profile.settings,
                         queued.message if queued.message else queued.payload,
                         outcome="OutboundTransportManager.DELIVER.END."
                         + queued.endpoint,
@@ -372,13 +436,13 @@ class OutboundTransportManager:
                     else:
                         queued.state = QueuedOutboundMessage.STATE_ENCODE
                         p_time = trace_event(
-                            self.context.settings,
+                            self.root_profile.settings,
                             queued.message if queued.message else queued.payload,
                             outcome="OutboundTransportManager.ENCODE.START",
                         )
                         self.encode_queued_message(queued)
                         trace_event(
-                            self.context.settings,
+                            self.root_profile.settings,
                             queued.message if queued.message else queued.payload,
                             outcome="OutboundTransportManager.ENCODE.END",
                             perf_counter=p_time,
@@ -400,20 +464,24 @@ class OutboundTransportManager:
 
     def encode_queued_message(self, queued: QueuedOutboundMessage) -> asyncio.Task:
         """Kick off encoding of a queued message."""
+
+        transport = self.get_transport_instance(queued.transport_id)
+
         queued.task = self.task_queue.run(
-            self.perform_encode(queued),
+            self.perform_encode(queued, transport.wire_format),
             lambda completed: self.finished_encode(queued, completed),
         )
         return queued.task
 
-    async def perform_encode(self, queued: QueuedOutboundMessage):
+    async def perform_encode(
+        self, queued: QueuedOutboundMessage, wire_format: BaseWireFormat = None
+    ):
         """Perform message encoding."""
-        transport = self.get_transport_instance(queued.transport_id)
-        wire_format = transport.wire_format or await queued.context.inject(
-            BaseWireFormat
-        )
+        wire_format = wire_format or self.root_profile.inject(BaseWireFormat)
+
+        session = await queued.profile.session()
         queued.payload = await wire_format.encode_message(
-            queued.context,
+            session,
             queued.message.payload,
             queued.target.recipient_keys,
             queued.target.routing_keys,
@@ -434,7 +502,13 @@ class OutboundTransportManager:
         """Kick off delivery of a queued message."""
         transport = self.get_transport_instance(queued.transport_id)
         queued.task = self.task_queue.run(
-            transport.handle_message(queued.context, queued.payload, queued.endpoint),
+            transport.handle_message(
+                queued.profile,
+                queued.payload,
+                queued.endpoint,
+                queued.metadata,
+                queued.api_key,
+            ),
             lambda completed: self.finished_deliver(queued, completed),
         )
         return queued.task
